import checks
import torch
import pandas as pd
import matplotlib.pyplot as plt

import distances.cpp
import distances.cuda

from distances import distances

import seaborn as sns
from tqdm.auto import tqdm

dtype = torch.float32

class Options:
    norm = 2
    eps = 1e-4
    runs = 100
    scale = 'µs'

options = Options()

DIMS = [16, 32, 64, 128, 256, 512]

state_sizes = []
devices = []
dims = []
means = []
whats = []

for _device in tqdm(['cuda', 'cpu'], desc='device', leave=False):
    for state_size in tqdm([32, 64, 128], desc='State size', leave=False):
        for dim in tqdm(DIMS, desc='dim', leave=False):
            options.state_size = state_size
            device = torch.device(_device)
            options.x1_size = dim
            options.x2_size = dim
            forward_min, forward_average, backward_min, backward_average = checks._benchmark(device, dtype, distances, options)

            for key, mean in (("forward", forward_average), ("backward", backward_average)):
                dims.append(dim)
                whats.append(key)
                state_sizes.append(state_size)
                devices.append(_device)
                means.append(mean)

data = pd.DataFrame({
    "state_size": state_sizes,
    "device": devices,
    "dim": dims,
    "mean": means,
    "what": whats
})


g = sns.FacetGrid(data, row="what", col="state_size", hue="device", margin_titles=True)
g.map(sns.lineplot, "dim", "mean", marker="o", ci=None).set(yscale ='log')
g.add_legend()
g.savefig("perf.pdf")
