import math
from torch import nn
from torch.autograd import Function
import torch

NTHREADS=32

class DistancesFunction(Function):
    @staticmethod
    def forward(ctx, input1, input2, p=2, eps=1e-6):
        assert input1.size(1) == input2.size(1), "La dimension des tenseurs ne correspond pas"

        if input1.is_cuda and input2.is_cuda:
            from .cuda import forward
            output = forward(NTHREADS, input1, input2, p, eps)
        else:
            from .cpp import forward
            output = forward(input1, input2, p, eps)

        ctx.save_for_backward(input1, input2, output, torch.tensor(p), torch.tensor(eps))
        return output

    @staticmethod
    def backward(ctx, grad_output):
        input1, input2, output, p, eps = ctx.saved_tensors
        p = float(p)
        eps = float(eps)

        if input1.is_cuda and input2.is_cuda:
            from .cuda import backward
            outputs = backward(NTHREADS, grad_output, output, input1, input2, p, eps)
        else:
            from .cpp import backward
            outputs = backward(grad_output, output, input1, input2, p, eps)

        return outputs[0], outputs[1], None, None


distances = DistancesFunction.apply
